import os
from tqdm import tqdm
from PIL import Image
import re
import ffmpeg
import cv2

from bokeh.io import export_png, export_svg
import glob

class VideoManager():
    """
    The VideoManager class provides functionalities to manage video and image files. 
    It allows the user to save video frames as images, generate a video from saved images,
    create a video with audio, and delete all saved images. It also provides options to 
    display a progress bar while generating images and to set the output file format for 
    images.

    Attributes:
        fretboard (Fretboard): The Fretboard object.
        video (Video): The Video object.
        images (Images): The Images object.
        imageProgressBar (bool): Flag indicating whether to display a progress bar while generating images.
    """
    def __init__(self, fretboard, video, images, imageProgressBar=True):
        self.fretboard = fretboard
        self.video = video
        self.images = images
        self.imageProgressBar = imageProgressBar

    def saveAsVideoImages(self):
        """
        Saves video frames as images.

        This method iterates over the frames of the video and saves each frame as an image.

        Raises:
            FileNotFoundError: If the output directory specified in the Images object does not exist.
        """
        oriImgName = self.images.name
        print(oriImgName)
        for k, v in self.video.getFramesItems():
            self.fretboard.updateFretboard(v)
            self.images.name = str(k)+oriImgName
            self.saveImage()
            print("saving"+self.images.name)
        print("done")

    # 
    def saveAsVideoImagesNoSeconds(self):
        """
        Saves video frames as images when the number of seconds is not defined.
        For guitarset and other data where number of second is not defined

        This method saves video frames as images without considering the number of seconds.
        It uses a dictionary to track previously saved frames to avoid duplicate saving.

        Raises:
            FileNotFoundError: If the output directory specified in the Images object does not exist.
        """
        oriImgName = self.images.name
        images = {}
        print("IMAGES Generateing")
        for i in tqdm(range(len(self.video.getFrames())), disable=not(self.imageProgressBar)):
            frame = self.video.getFrames()[i]
            self.images.name = str(i)+oriImgName

            if frame in images:
                image = images[frame]
                image.copy().save(os.path.join(
                    self.images.outputPathName, self.images.name + self.images.meta))
            else:
                self.fretboard.updateFretboard(self.video.getFrames()[i])
                self.saveImage()
                image = Image.open(os.path.join(
                    self.images.outputPathName, self.images.name + self.images.meta))
                images[frame] = image.copy()
        print("IMAGES Generate done")

    def saveAsVideo(self):
        """
        Generates a video from saved images.

        This method reads the saved images, orders them based on their filenames, and
        creates a video from the ordered frames.

        Raises:
            FileNotFoundError: If the output directory specified in the Images object does not exist.
        """
        images = os.listdir(self.images.outputPathName)
        images = sorted(images, key=lambda s: [
                        int(x) if x.isdigit() else x for x in re.split('(\d+)', s)])

        fourcc = cv2.VideoWriter_fourcc(*self.video.getCodec())
        frameSize = (self.fretboard.fretboardFig.fig.width, self.fretboard.fretboardFig.fig.height)

        videoWriter = cv2.VideoWriter(self.video.getVideoPathWithName(
        )+self.video.getFileExtension(), fourcc, self.video.getFrameRate(), frameSize)

        for image in images:
            frame = cv2.imread(os.path.join(self.images.outputPathName, image))
            videoWriter.write(frame)

        cv2.destroyAllWindows()
        videoWriter.release()

        print("VIDEO " + self.video.getVideoName() +
              " saved at "+self.video.getVideoPathName())

    def createVideoWithAudio(self):
        """
        Creates a video with audio.

        This method combines the video file and audio file into a single video file with audio.

        Raises:
            FileNotFoundError: If the video or audio files specified in the Video object do not exist.
        """
        self.saveAsVideo()

        videoPath = ffmpeg.input(
            self.video.getVideoPathWithName()+self.video.getFileExtension())
        audioPath = ffmpeg.input(self.video.getAudioPathWithName())


        ffmpeg.concat(videoPath, audioPath, v=1, a=1).output(
            self.video.getVideoWAudioPathWithName()).run(overwrite_output=True)
        print("video save with audio done")

    def saveVideoWithAudio(self):
        """
        Saves a video with audio.

        This method combines the video file and audio file into a single video file with audio
        and saves it with a specified name.

        Raises:
            FileNotFoundError: If the video or audio files specified in the Video object do not exist.
        """
        videoPath = ffmpeg.input(
            self.video.getVideoPathWithName()+self.video.getFileExtension())
        audioPath = ffmpeg.input(self.video.getAudioPathWithName())

        ffmpeg.concat(videoPath, audioPath, v=1, a=1).output(
            self.video.getVideoName(
            )+".mp4").run(overwrite_output=True)
        print("video save with audio done")
    
    def saveImage(self):
        """
        Saves the fretboard visualization as an image.

        This method saves the current state of the fretboard visualization as an image file.
        The image format is determined by the file extension specified in the Images object.
        png and svg for now.

        Raises:
            FileNotFoundError: If the output directory specified in the Images object does not exist.
        """
        if (self.images.meta.lower() == ".png"):
            export_png(self.fretboard.getFretboardFig().fig, filename=self.images.fileName)

        elif (self.images.meta.lower() == ".svg"):
            export_svg(self.fretboard.getFretboardFig().fig, filename=self.images.fileName)


    def deleteAllImages(self):
        """
        Deletes all saved images.

        This method deletes all image files in the output directory specified in the Images object.

        Raises:
            FileNotFoundError: If the output directory specified in the Images object does not exist.
        """
        files = glob.glob(os.path.join(self.images.outputPathName, "*"))
        for f in files:
            os.remove(f)
        print("All Images Delete")

        
    @property
    def imageProgressBar(self):
        """
        bool: Flag indicating whether to display a progress bar while generating images.
        """
        return self._imageProgressBar

    @imageProgressBar.setter
    def imageProgressBar(self, imageProgressBar):
        """
        Setter for the imageProgressBar property.

        Args:
            imageProgressBar (bool): Flag indicating whether to display a progress bar while generating images.
        """
        self._imageProgressBar = imageProgressBar


from seeFretboard.Utilities.Constants import STANDARD_TUNING, STANDARD_TUNING_MIDI
from seeFretboard.Designs.CirlceNote import CircleNote

"""
These classes can be used to configure and customize the settings of a guitar fretboard for various purposes.
"""

class FretboardTheme:
    """Represents a theme for the fretboard visualization."""

    def __init__(self, theme=None, **kwargs):
        """
        Initializes a FretboardTheme object.

        Parameters:
            theme (str): The theme to apply ("blue, dark, wood, or green"). If None, a custom theme (light basically) is applied.
            **kwargs: Additional keyword arguments for custom theme settings.
        """

        self.customTheme(**kwargs)
        if(theme == "blue"):
            self.BlueTheme()
        elif(theme== "dark"):
            self.darkTheme()
        elif (theme == "wood"):
            self.woodTheme()
        elif(theme == "green"):
            self.greenTheme()

    def customTheme(self, **kwargs):
        """
        Applies a custom theme to the FretboardTheme object.

        Parameters:
            **kwargs: Keyword arguments for custom theme settings.
        """
        orientation = kwargs.get("orientation", "h")
        if isinstance(orientation, FretboardOrientation):
            self.orientation = orientation
        else:
            self.orientation = FretboardOrientation(orientation)

        tuning = kwargs.get("tuning")
        if tuning is not None and not isinstance(tuning, Tuning):
            raise TypeError("Invalid tuning input. Expected instance of Tuning.")
        if isinstance(tuning, Tuning):
            self.tuning = tuning
        else:
            self.tuning = Tuning(**kwargs)
            
        fretboardRange = kwargs.get("fretboardRange")
        if isinstance(fretboardRange, FretboardRange):
            self.fretboardRange = fretboardRange
        else:
            self.fretboardRange = FretboardRange(kwargs.get("fretFrom", 1),
                                                    kwargs.get("fretTo", 12),
                                                     kwargs.get("numOfString", 6))

        
        fretboardDesign = kwargs.get("fretboardDesign")
        if fretboardDesign is not None and not isinstance(fretboardDesign, FretboardDesign):
            raise TypeError("Invalid fretboardDesign input. Expected instance of FretboardDesign.")
        if isinstance(fretboardDesign, FretboardDesign):
            self.fretboardDesign = fretboardDesign
        else:
            self.fretboardDesign = FretboardDesign(**kwargs)
    
    
    def BlueTheme(self):
        self.fretboardDesign.stringsColor = "#5188D4"
        self.fretboardDesign.fretColor = "#5188D4"
        self.fretboardDesign.fretboardMarkerColor = "#F2F9FC"
        self.fretboardDesign.backgroundColor = "#ADC7EC"
        self.fretboardDesign.setNoteType("green")
        self.fretboardDesign.tuningLabelColor = "white"
        self.fretboardDesign.fretLabelColor = "white"
        

    def darkTheme(self):
        self.fretboardDesign.stringsColor = "black"
        self.fretboardDesign.fretColor = "black"
        self.fretboardDesign.fretboardMarkerColor = "black"
        self.fretboardDesign.backgroundColor = "#001f3f"
        self.fretboardDesign.setNoteType("green")
        self.fretboardDesign.tuningLabelColor = "white"
        self.fretboardDesign.fretLabelColor = "white"

    def woodTheme(self):
        self.fretboardDesign.stringsColor = "#DFFFED"
        self.fretboardDesign.fretColor = "#DFFFED"
        self.fretboardDesign.fretboardMarkerColor = "#223447"
        self.fretboardDesign.backgroundColor = "#63462D"
        self.fretboardDesign.setNoteType("black")
        self.fretboardDesign.tuningLabelColor = "white"
        self.fretboardDesign.fretLabelColor = "white"

    def greenTheme(self):
        self.fretboardDesign.stringsColor = "#001f3f"
        self.fretboardDesign.fretColor = "#001f3f"
        self.fretboardDesign.fretboardMarkerColor = "#1F2421"
        self.fretboardDesign.backgroundColor = "#DFFFED"
        self.fretboardDesign.setNoteType("darkBlue")

class FretboardOrientation:
    """Represents the orientation (horizontal or vertical) of the fretboard."""

    def __init__(self, orientation="h"):
        """
        Initializes a FretboardOrientation object.

        Parameters:
            orientation (str): The orientation of the fretboard ("horizontal" or "vertical").
                               Defaults to "horizontal" if not specified.
        """
        self.orientation = orientation
    
    @property
    def orientation(self):
        """
        Get the orientation of the fretboard.

        Returns:
            str: The orientation of the fretboard.
        """
        return self._orientation
    
    @orientation.setter
    def orientation(self, orientation):
        """
        Set the orientation of the fretboard.

        Parameters:
            orientation (str): The orientation of the fretboard.
                               Must be "horizontal" ("h") or "vertical" ("v").
        """
        if orientation is None or orientation.lower() not in ["horizontal", "h", "vertical", "v"]:
            raise ValueError("Invalid orientation value. Must be 'horizontal', 'h', 'vertical', or 'v'.")
        self._orientation = orientation.lower()

class Tuning:
    """Represents the tuning of the strings on the fretboard.
       """

    def __init__(self, **kwargs):
        """
        Initializes a Tuning object.

        Parameters:
            **kwargs: Keyword arguments for tuning settings.
            letterTuning (list) A list of letter based tuning
            midiTuning (list) A list of midi based tuning
            numOfString (int) Number of Strings on Guitar
        """
        self.letterTuning = kwargs.get("letterTuning", STANDARD_TUNING)
        self.midiTuning = kwargs.get("midiTuning", STANDARD_TUNING_MIDI)
        self.numOfStrings = kwargs.get("numOfStrings", 6)
        
    @property
    def letterTuning(self):
        """
        Get the letter-based tuning of the strings.

        Returns:
            list: The letter-based tuning of the strings.
        """
        return self._letterTuning
    
    @letterTuning.setter
    def letterTuning(self, value):
        """
        Set the letter-based tuning of the strings.

        Parameters:
            value (list): The letter-based tuning of the strings.
        """
        self._letterTuning = value
        
    @property
    def midiTuning(self):
        """
        Get the MIDI note-based tuning of the strings.

        Returns:
            list: The MIDI note-based tuning of the strings.
        """
        return self._midiTuning
    
    @midiTuning.setter
    def midiTuning(self, value):
        """
        Set the MIDI note-based tuning of the strings.

        Parameters:
            value (list): The MIDI note-based tuning of the strings.
        """
        self._midiTuning = value
        
    @property
    def numOfStrings(self):
        """
        Get the number of strings on the fretboard.

        Returns:
            int: The number of strings on the fretboard.
        """
        return self._numOfStrings
    
    @numOfStrings.setter
    def numOfStrings(self, value):
        """
        Set the number of strings on the fretboard.

        Parameters:
            value (int): The number of strings on the fretboard.
        """
        self._numOfStrings = value

class FretboardRange:
    """Represents the range of frets and strings on the fretboard."""

    def __init__(self, fretFrom=1, fretTo=12, numOfString=6):
        """
        Initializes a FretboardRange object.

        Parameters:
            fretFrom (int): The starting fret number.
            fretTo (int): The ending fret number.
            numOfString (int): The number of strings on the fretboard.
        """
        if fretFrom <= 0:
            raise ValueError("fretFrom must be a positive integer.")
        if fretTo <= fretFrom:
            raise ValueError("fretTo must be greater than fretFrom.")
        if numOfString <= 0:
            raise ValueError("numOfString must be a positive integer.")
        self.fretFrom = fretFrom
        self.fretTo = fretTo
        self.numOfFrets = fretTo - fretFrom
        self.numOfStrings = numOfString
    
    @property
    def fretFrom(self):
        """
        Get the starting fret number.

        Returns:
            int: The starting fret number.
        """
        return self._fretFrom

    @fretFrom.setter
    def fretFrom(self, newFretFrom):
        """
        Set the starting fret number.

        Parameters:
            newFretFrom (int): The new starting fret number.
        """
        self._fretFrom = newFretFrom

    @property
    def fretTo(self):
        """
        Get the ending fret number.

        Returns:
            int: The ending fret number.
        """
        return self._fretTo

    @fretTo.setter
    def fretTo(self, newFretTo):
        """
        Set the ending fret number.

        Parameters:
            newFretTo (int): The new ending fret number.
        """
        self._fretTo = newFretTo

    @property
    def numOfFrets(self):
        """
        Get the number of frets in the range.

        Returns:
            int: The number of frets in the range.
        """
        return int(self.fretTo - self.fretFrom)

    @numOfFrets.setter
    def numOfFrets(self, numOfFrets):
        """
        Set the number of frets in the range.

        Parameters:
            numOfFrets (int): The new number of frets in the range.
        """
        self._numOfFrets = numOfFrets

    @property
    def numOfStrings(self):
        """
        Get the number of strings on the fretboard.

        Returns:
            int: The number of strings on the fretboard.
        """
        return self._numOfStrings

    @numOfStrings.setter
    def numOfStrings(self, numOfStrings):
        """
        Set the number of strings on the fretboard.

        Parameters:
            numOfStrings (int): The new number of strings on the fretboard.
        """
        self._numOfStrings = numOfStrings

class FretboardDesign:
    """Represents the design settings of the fretboard."""

    def __init__(self, **kwargs):
        """
        Initializes a FretboardDesign object.

        Parameters:
            **kwargs: Keyword arguments for design settings.
            showTuning (bool): Determines whether the tuning of the strings is displayed on the fretboard. Default is True.
            showFretboardNumber (bool): Determines whether the fretboard number is displayed on the fretboard. Default is True.
            toolBar (str): Determines whether a toolbar is displayed on the fretboard. "right" is default and means display on right side. None means don't display. Other Options: 'above', 'below', 'left'
            distanceBetweenFrets (int): The distance between two consecutive frets on the fretboard. Default is 5.
            distanceBetweenStrings (int): The distance between two consecutive strings on the fretboard. Default is 2.
            fretColor (str): The color of the frets on the fretboard. Default is "black".
            fretThinkness (int): The thickness of the fret lines on the fretboard. Default is 1.
            stringsColor (str): The color of the strings on the fretboard. Default is "black".
            stringThinkness (float): The thickness of the string lines on the fretboard. Default is 1.5.
            fretOpacity (float): The opacity of the frets on the fretboard. Default is 0.3.
            stringsOpacity (float): The opacity of the strings on the fretboard. Default is 1.0.
            backgroundColor (str): The background color of the fretboard. Default is "white".
            fretboardMarkerColor (str): The color of the fretboard markers. Default is "#DCDCDC".
        """
        # display
        self.showTuning = kwargs.get("showTuning", True)
        self.showFretboardNumber = kwargs.get("showFretboardNumber", True)
        self.toolBar = kwargs.get("toolBar", "right")

        # fretboard design
        self.distanceBetweenFrets = kwargs.get("distanceBetweenFrets", 5)
        self.distanceBetweenStrings = kwargs.get("distanceBetweenStrings", 2)
        self.fretColor = kwargs.get("fretColor", "black")
        self.fretThinkness = kwargs.get("stringThinkness", 1)
        self.stringsColor = kwargs.get("stringsColor", "black")
        self.stringThinkness = kwargs.get("stringThinkness", 1.5)
        self.fretOpacity = kwargs.get("fretOpacity", 0.3)
        self.stringsOpacity = kwargs.get("stringsOpacity", 1)

        self.backgroundColor = kwargs.get("backgroundColor", "white")
        self.fretboardMarkerColor = kwargs.get("fretboardMarkerColor", "#DCDCDC")

        #notes colors
        self.noteType = "prediction"

        self.noteTypes = {
            'prediction': CircleNote(), #default using that
            'groundTruth': CircleNote(noteFaceColor="red"),
            'white':CircleNote(noteFaceColor="white", noteTextColor = "black"),
            'black':CircleNote(noteFaceColor="black", noteTextColor="white",noteEdgeColor="#DFFFED", noteEdgeWidth=1), 
            'darkBlue':CircleNote(noteFaceColor="#001f3f", noteTextColor="#DFFFED"),
            'green':CircleNote(noteFaceColor="#DFFFED",noteTextColor="#001f3f",noteEdgeColor="#white" ),
        }

        self.tuningLabelColor = "black"
        self.tuningLabelFontSize = "10pt"
        self.fretLabelColor = "black"
        self.fretLabelFontsize = "10pt"

    @property
    def backgroundColor(self):
        """
        Get the background color.

        Returns:
            str: The background color.
        """
        return self._backgroundColor

    @backgroundColor.setter
    def backgroundColor(self, value):
        """
        Set the background color.

        Args:
            value (str): The background color to set.
        """
        self._backgroundColor = value

    @property
    def showTuning(self):
        """
        Get the flag indicating whether to display tuning information.

        Returns:
            bool: True if tuning information should be displayed, False otherwise.
        """
        return self._showTuning

    @showTuning.setter
    def showTuning(self, value):
        """
        Set the flag indicating whether to display tuning information.

        Parameters:
            value (bool): True to display tuning information, False otherwise.
        """
        self._showTuning = value

    @property
    def showFretboardNumber(self):
        """
        Get the flag indicating whether to display fretboard numbers.

        Returns:
            bool: True if fretboard numbers should be displayed, False otherwise.
        """
        return self._showFretboardNumber

    @showFretboardNumber.setter
    def showFretboardNumber(self, value):
        """
        Set the flag indicating whether to display fretboard numbers.

        Parameters:
            value (bool): True to display fretboard numbers, False otherwise.
        """
        self._showFretboardNumber = value

    @property
    def toolBar(self):
        """
        Get the position indicating whether to display the toolbar.

        Returns:
            str: None, right, left, above, below
        """
        return self._toolBar

    @toolBar.setter
    def toolBar(self, value):
        """
        Set the position indicating whether to display the toolbar.

        Parameters:
            value (str): None, right, left, above, below
        """
        self._toolBar = value

    @property
    def distanceBetweenFrets(self):
        """
        Get the distance between frets.

        Returns:
            float: The distance between frets.
        """
        return self._distanceBetweenFrets

    @distanceBetweenFrets.setter
    def distanceBetweenFrets(self, value):
        """
        Set the distance between frets.

        Parameters:
            value (float): The distance between frets.
        """
        self._distanceBetweenFrets = value

    @property
    def distanceBetweenStrings(self):
        """
        Get the distance between strings.

        Returns:
            float: The distance between strings.
        """
        return self._distanceBetweenStrings

    @distanceBetweenStrings.setter
    def distanceBetweenStrings(self, value):
        """
        Set the distance between strings.

        Parameters:
            value (float): The distance between strings.
        """
        self._distanceBetweenStrings = value

    @property
    def fretColor(self):
        """
        Get the color of the frets.

        Returns:
            str: The color of the frets.
        """
        return self._fretColor

    @fretColor.setter
    def fretColor(self, value):
        """
        Set the color of the frets.

        Parameters:
            value (str): The color of the frets.
        """
        self._fretColor = value

    @property
    def stringsColor(self):
        """
        Get the color of the strings.

        Returns:
            str: The color of the strings.
        """
        return self._stringsColor

    @stringsColor.setter
    def stringsColor(self, value):
        """
        Set the color of the strings.

        Parameters:
            value (str): The color of the strings.
        """
        self._stringsColor = value

    @property
    def fretOpacity(self):
        """
        Get the opacity of the frets.

        Returns:
            float: The opacity of the frets.
        """
        return self._fretOpacity

    @fretOpacity.setter
    def fretOpacity(self, value):
        """
        Set the opacity of the frets.

        Parameters:
            value (float): The opacity of the frets.
        """
        self._fretOpacity = value

    @property
    def stringsOpacity(self):
        """
        Get the opacity of the strings.

        Returns:
            float: The opacity of the strings.
        """
        return self._stringsOpacity

    @stringsOpacity.setter
    def stringsOpacity(self, value):
        """
        Set the opacity of the strings.

        Parameters:
            value (float): The opacity of the strings.
        """
        self._stringsOpacity = value

    @property
    def fretboardMarkerColor(self):
        """
        Get the color of the fretboard markers.

        Returns:
            str: The color of the fretboard markers.
        """
        return self._fretboardMarkerColor

    @fretboardMarkerColor.setter
    def fretboardMarkerColor(self, value):
        """
        Set the color of the fretboard markers.

        Parameters:
            value (str): The color of the fretboard markers.
        """
        self._fretboardMarkerColor = value

    @property
    def stringThinkness(self):
        """Get the thickness of the strings."""
        return self._stringThinkness
    
    @stringThinkness.setter
    def stringThinkness(self, value):
        """Set the thickness of the strings.

        Parameters:
            value (float): The thickness of the strings.
        """
        self._stringThinkness = value

    @property
    def fretThinkness(self):
        """Get the thickness of the frets."""
        return self._fretThinkness

    @fretThinkness.setter
    def fretThinkness(self, value):
        """Set the thickness of the frets.

        Parameters:
            value (float): The thickness of the frets.
        """
        self._fretThinkness = value

    def getNoteTypes(self):
        """Get the note types dictionary."""
        return self.noteTypes
    
    def getNoteType(self, key):
        """Get the note types associated with a specific key."""
        return self.noteTypes.get(key)

    def setNoteTypes(self, key, value):
        """Set the note types for a specific key."""
        self.noteTypes[key] = value

    def getCurrentNoteType(self):
        """Get the current note type."""
        return self.noteType

    def setNoteType(self, noteType):
        """Set the current note type."""
        self.noteType = noteType

    def getCurrentNoteTypeValue(self):
        """Get the note types associated with the current note type."""
        return self.getNoteType(self.getCurrentNoteType())

    @property
    def tuningLabelColor(self):
        """Get the color of the tuning labels."""
        return self._tuningLabelColor

    @tuningLabelColor.setter
    def tuningLabelColor(self, color):
        """Set the color of the tuning labels."""
        self._tuningLabelColor = color

    @property
    def tuningLabelFontSize(self):
        """Get the font size of the tuning labels."""
        return self._tuningLabelFontSize

    @tuningLabelFontSize.setter
    def tuningLabelFontSize(self, size):
        """Set the font size of the tuning labels."""
        self._tuningLabelFontSize = size

    @property
    def fretLabelColor(self):
        """Get the color of the fret labels."""
        return self._fretLabelColor

    @fretLabelColor.setter
    def fretLabelColor(self, color):
        """Set the color of the fret labels."""
        self._fretLabelColor = color

    @property
    def fretLabelFontSize(self):
        """Get the font size of the fret labels."""
        return self._fretLabelFontSize

    @fretLabelFontSize.setter
    def fretLabelFontSize(self, size):
        """Set the font size of the fret labels."""
        self._fretLabelFontSize = size


import os
import sys
sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), '..')))

from seeFretboard.Videos.Frame import Frame
from seeFretboard.Videos.Video import Video
from seeFretboard.Videos.TabSequence import TabSequence
from seeFretboard import SeeFretboard


fretboard = SeeFretboard("v", 1, 12)
tabSeq = TabSequence(8)
video = Video(0, 0, 0, 0)
fretboard.drawVerticalFretboard()

video.setVideoPathName(os.getcwd())
video.setVideoName("video_00_BN3-119-G_comp_hex_cln")
video.setAudioPathName(os.getcwd())
video.setAudioName("00_BN3-119-G_comp_hex_cln.wav")
fretboard.setVideo(video)
fretboard.createVideoWithAudio("videoWithAudio_00_BN3-119-G_comp_hex_cln")



import os
import sys
sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), '..')))

from seeFretboard import SeeFretboard

fretboard = SeeFretboard("h", 1, 12)
fretboard.drawFretboard("v")#this function can change from h to v

#fretboard.getPitchCollection().setPitchesType("pitchesScaleDegrees")

"""
Drop2 String6
"""
fretboard.addDropChord("c","maj7")
# fretboard.setNoteType("groundTruth")
# fretboard.addDropChord("c", "maj7","Drop2", "2")
# fretboard.setNoteType("prediction")
# fretboard.addDropChord("c", "maj7","Drop2", "3")
# fretboard.setNoteType("groundTruth")
# fretboard.addDropChord("c", "maj7","Drop2", "4")

# fretboard.addDropChord("c","dom7")
# fretboard.setNoteType("groundTruth")
# fretboard.addDropChord("c", "dom7","Drop2", "2")
# fretboard.setNoteType("prediction")
# fretboard.addDropChord("c", "dom7","Drop2", "3")
# fretboard.setNoteType("groundTruth")
# fretboard.addDropChord("c", "dom7","Drop2", "4")

# fretboard.addDropChord("c","min7")
# fretboard.setNoteType("groundTruth")
# fretboard.addDropChord("c", "min7","Drop2", "2")
# fretboard.setNoteType("prediction")
# fretboard.addDropChord("c", "min7","Drop2", "3")
# fretboard.setNoteType("groundTruth")
# fretboard.addDropChord("c", "min7","Drop2", "4")

# fretboard.addDropChord("c","min7b5")
# fretboard.setNoteType("groundTruth")
# fretboard.addDropChord("c", "min7b5","Drop2", "2")
# fretboard.setNoteType("prediction")
# fretboard.addDropChord("c", "min7b5","Drop2", "3")
# fretboard.setNoteType("groundTruth")
# fretboard.addDropChord("c", "min7b5","Drop2", "4")

# fretboard.addDropChord("c","dim7")
# fretboard.setNoteType("groundTruth")
# fretboard.addDropChord("c", "dim7","Drop2", "2")
# fretboard.setNoteType("prediction")
# fretboard.addDropChord("c", "dim7","Drop2", "3")
# fretboard.setNoteType("groundTruth")
# fretboard.addDropChord("c", "dim7","Drop2", "4")
"""
Drop2 String5
"""
# fretboard.addDropChord("c","maj7","Drop2","1","5")
# fretboard.setNoteType("groundTruth")
# fretboard.addDropChord("c", "maj7","Drop2", "2","5")
# fretboard.setNoteType("prediction")
# fretboard.addDropChord("c", "maj7","Drop2", "3","5")
# fretboard.setNoteType("groundTruth")
# fretboard.addDropChord("c", "maj7","Drop2", "4","5")

# fretboard.addDropChord("c","dom7","Drop2","1","5")
# fretboard.setNoteType("groundTruth")
# fretboard.addDropChord("c", "dom7","Drop2", "2","5")
# fretboard.setNoteType("prediction")
# fretboard.addDropChord("c", "dom7","Drop2", "3","5")
# fretboard.setNoteType("groundTruth")
# fretboard.addDropChord("c", "dom7","Drop2", "4","5")

# fretboard.addDropChord("c","min7","Drop2","1","5")
# fretboard.setNoteType("groundTruth")
# fretboard.addDropChord("c", "min7","Drop2", "2","5")
# fretboard.setNoteType("prediction")
# fretboard.addDropChord("c", "min7","Drop2", "3","5")
# fretboard.setNoteType("groundTruth")
# fretboard.addDropChord("c", "min7","Drop2", "4","5")

# fretboard.addDropChord("c","min7b5","drop2","1","5")
# fretboard.setNoteType("groundTruth")
# fretboard.addDropChord("c", "min7b5","Drop2", "2","5")
# fretboard.setNoteType("prediction")
# fretboard.addDropChord("c", "min7b5","Drop2", "3","5")
# fretboard.setNoteType("groundTruth")
# fretboard.addDropChord("c", "min7b5","Drop2", "4","5")

# fretboard.addDropChord("c","dim7","drop2","1","5")
# fretboard.setNoteType("groundTruth")
# fretboard.addDropChord("c", "dim7","Drop2", "2","5")
# fretboard.setNoteType("prediction")
# fretboard.addDropChord("c", "dim7","Drop2", "3","5")
# fretboard.setNoteType("groundTruth")
# fretboard.addDropChord("c", "dim7","Drop2", "4","5")


"""
Drop2 String4
"""
# fretboard.addDropChord("c","maj7","Drop2","1","4")
# fretboard.setNoteType("groundTruth")
# fretboard.addDropChord("c", "maj7","Drop2", "2","4")
# fretboard.setNoteType("prediction")
# fretboard.addDropChord("c", "maj7","Drop2", "3","4")
# fretboard.setNoteType("groundTruth")
# fretboard.addDropChord("c", "maj7","Drop2", "4","4")

# fretboard.addDropChord("c","dom7","Drop2","1","4")
# fretboard.setNoteType("groundTruth")
# fretboard.addDropChord("c", "dom7","Drop2", "2","4")
# fretboard.setNoteType("prediction")
# fretboard.addDropChord("c", "dom7","Drop2", "3","4")
# fretboard.setNoteType("groundTruth")
# fretboard.addDropChord("c", "dom7","Drop2", "4","4")

# fretboard.addDropChord("c","min7","Drop2","1","4")
# fretboard.setNoteType("groundTruth")
# fretboard.addDropChord("c", "min7","Drop2", "2","4")
# fretboard.setNoteType("prediction")
# fretboard.addDropChord("c", "min7","Drop2", "3","4")
# fretboard.setNoteType("groundTruth")
# fretboard.addDropChord("c", "min7","Drop2", "4","4")

# fretboard.addDropChord("c","min7b5","drop2","1","4")
# fretboard.setNoteType("groundTruth")
# fretboard.addDropChord("c", "min7b5","Drop2", "2","4")
# fretboard.setNoteType("prediction")
# fretboard.addDropChord("c", "min7b5","Drop2", "3","4")
# fretboard.setNoteType("groundTruth")
# fretboard.addDropChord("c", "min7b5","Drop2", "4","4")

# fretboard.addDropChord("c","dim7","drop2","1","4")
# fretboard.setNoteType("groundTruth")
# fretboard.addDropChord("c", "dim7","Drop2", "2","4")
# fretboard.setNoteType("prediction")
# fretboard.addDropChord("c", "dim7","Drop2", "3","4")
# fretboard.setNoteType("groundTruth")
# fretboard.addDropChord("c", "dim7","Drop2", "4","4")

"""
Drop3 String6
"""
# fretboard.addDropChord("c","maj7","Drop3","1","6")
# fretboard.setNoteType("groundTruth")
# fretboard.addDropChord("c", "maj7","Drop3", "2","6")
# fretboard.setNoteType("prediction")
# fretboard.addDropChord("c", "maj7","Drop3", "3","6")
# fretboard.setNoteType("groundTruth")
# fretboard.addDropChord("c", "maj7","Drop3", "4","6")

# fretboard.addDropChord("c","dom7","Drop3","1","6")
# fretboard.setNoteType("groundTruth")
# fretboard.addDropChord("c", "dom7","Drop3", "2","6")
# fretboard.setNoteType("prediction")
# fretboard.addDropChord("c", "dom7","Drop3", "3","6")
# fretboard.setNoteType("groundTruth")
# fretboard.addDropChord("c", "dom7","Drop3", "4","6")

# fretboard.addDropChord("c","min7","Drop3","1","6")
# fretboard.setNoteType("groundTruth")
# fretboard.addDropChord("c", "min7","Drop3", "2","6")
# fretboard.setNoteType("prediction")
# fretboard.addDropChord("c", "min7","Drop3", "3","6")
# fretboard.setNoteType("groundTruth")
# fretboard.addDropChord("c", "min7","Drop3", "4","6")

# fretboard.addDropChord("c","min7b5","Drop3","1","6")
# fretboard.setNoteType("groundTruth")
# fretboard.addDropChord("c", "min7b5","Drop3", "2","6")
# fretboard.setNoteType("prediction")
# fretboard.addDropChord("c", "min7b5","Drop3", "3","6")
# fretboard.setNoteType("groundTruth")
# fretboard.addDropChord("c", "min7b5","Drop3", "4","6")

# fretboard.addDropChord("c","dim7","Drop3","1","6")
# fretboard.setNoteType("groundTruth")
# fretboard.addDropChord("c", "dim7","Drop3", "2","6")
# fretboard.setNoteType("prediction")
# fretboard.addDropChord("c", "dim7","Drop3", "3","6")
# fretboard.setNoteType("groundTruth")
# fretboard.addDropChord("c", "dim7","Drop3", "4","6")
"""
Drop3 String5
"""
# fretboard.addDropChord("c","maj7","Drop3","1","5")
# fretboard.setNoteType("groundTruth")
# fretboard.addDropChord("c", "maj7","Drop3", "2","5")
# fretboard.setNoteType("prediction")
# fretboard.addDropChord("c", "maj7","Drop3", "3","5")
# fretboard.setNoteType("groundTruth")
# fretboard.addDropChord("c", "maj7","Drop3", "4","5")

# fretboard.addDropChord("c","dom7","Drop3","1","5")
# fretboard.setNoteType("groundTruth")
# fretboard.addDropChord("c", "dom7","Drop3", "2","5")
# fretboard.setNoteType("prediction")
# fretboard.addDropChord("c", "dom7","Drop3", "3","5")
# fretboard.setNoteType("groundTruth")
# fretboard.addDropChord("c", "dom7","Drop3", "4","5")

# fretboard.addDropChord("c","min7","Drop3","1","5")
# fretboard.setNoteType("groundTruth")
# fretboard.addDropChord("c", "min7","Drop3", "2","5")
# fretboard.setNoteType("prediction")
# fretboard.addDropChord("c", "min7","Drop3", "3","5")
# fretboard.setNoteType("groundTruth")
# fretboard.addDropChord("c", "min7","Drop3", "4","5")

# fretboard.addDropChord("c","min7b5","Drop3","1","5")
# fretboard.setNoteType("groundTruth")
# fretboard.addDropChord("c", "min7b5","Drop3", "2","5")
# fretboard.setNoteType("prediction")
# fretboard.addDropChord("c", "min7b5","Drop3", "3","5")
# fretboard.setNoteType("groundTruth")
# fretboard.addDropChord("c", "min7b5","Drop3", "4","5")

# fretboard.addDropChord("c","dim7","Drop3","1","5")
# fretboard.theme.fretboardDesign.setNoteType("groundTruth")
# fretboard.addDropChord("c", "dim7","Drop3", "2","5")
# fretboard.theme.fretboardDesign.setNoteType("prediction")
# fretboard.addDropChord("c", "dim7","Drop3", "3","5")
# fretboard.theme.fretboardDesign.setNoteType("groundTruth")
# fretboard.addDropChord("c", "dim7","Drop3", "4","5")

fretboard.showFretboard()


import os
import sys
sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), '..')))

from seeFretboard.Videos.Frame import Frame
from seeFretboard.Videos.Video import Video
from seeFretboard.Videos.TabSequence import TabSequence
from seeFretboard import SeeFretboard
from seeFretboard.Videos import Video, TabSequence, Audio, VideoManager, Images


fretboard = SeeFretboard("v", 1, 12)
tabSeq = TabSequence(8)
video = Video(0, 0, 0, 0)
fretboard.drawVerticalFretboard()
video.setVideoName("OpenStringG")

"""
Use guitar set track 0 to make frames
Then frames to images
then images to video with audio
"""
tabSeq.makingFrames()
video.setAudioName("00_BN3-119-G_comp_hex_cln.wav")

videoManager = VideoManager(fretboard,video,Images())
#videoManager.saveAsVideoImagesNoSeconds()#generate images for audio
videoManager.createVideoWithAudio()#createVideoWithMidiAudio



from setuptools import setup, find_packages

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setup(
    name='seeFretboard',
    version='0.1.5.4',
    author='Linda Rong Zhang',
    author_email='ronglindaz@gmail.com',
    description='For Release',
    long_description=long_description,
    long_description_content_type="text/markdown",
    url='https://github.com/LindaRZhang/seeFretboard',
    project_urls={
        "Bug Tracker": "https://github.com/LindaRZhang/seeFretboard/issues"
    },
    license='GNU',
    packages=find_packages(include=['seeFretboard', 'seeFretboard.*']),
    package_data={
        'seeFretboard': ['Images/logo.png',
                         'Outputs/Audios/*',
                         'Outputs/Embeds/*',
                         'Outputs/Images/*',
                         'Outputs/Videos/*'
                        ],
    },
    install_requires=['requests'],
    include_package_data=True
)



import numpy as np

from bokeh.io import curdoc, show
from bokeh.models import ColumnDataSource, Grid, LinearAxis, Plot, Text
import os
import sys
sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), '..')))

from seeFretboard import SeeFretboard

fretboard = SeeFretboard("v", 1, 12)
fretboard.drawVerticalFretboard()
fretboard.addNotesAllString("0,0,2,2,0,0")
fretboard.showFretboard()


N = 9
x = np.linspace(-2, 2, N)
y = x**2
a = "abcdefghijklmnopqrstuvwxyz"
text = [a[i*3:i*3+3] for i in range(N)]


plot = Plot(
    title=None, width=300, height=300,
    min_border=0, toolbar_location=None)

glyph = Text(x="x", y="y", text="text", angle=0.3, text_color="#96deb3")
plot.add_glyph(glyph)

# xaxis = LinearAxis()
# plot.add_layout(xaxis, 'below')

axis = LinearAxis()
# plot.add_layout(yaxis, 'left')

# plot.add_layout(Grid(dimension=0, ticker=xaxis.ticker))
# plot.add_layout(Grid(dimension=1, ticker=yaxis.ticker))

curdoc().add_root(plot)

show(plot)



from .Audio import Audio
from .Frame import Frame
from .TabSequence import TabSequence
from .VideoManager import VideoManager
from .VideoNote import VideoNote
from .Video import Video
from .Images import Images
from .Audio import Audio
from .Frame import Frame




class Frame:
    """
    The Frame class is designed to represent a frame in a video. 
    It stores the frame rate and frame period, which are important parameters
    for determining the timing and smoothness of the animation. 
    The main functionalities of the class are to set and get the frame rate 
    and frame period, which can be used to control the speed and duration of the video.

    Args:
        frameRate (int or float): The frame rate of the frame.

    Attributes:
        frameRate (int or float): The frame rate of the frame.
        framePeriod (float): The time duration of each frame in seconds.
    """

    def __init__(self, frameRate):
        """
        Initializes a Frame object with the specified frame rate.

        Args:
            frameRate (int or float): The frame rate of the frame.
        """
        self.frameRate = frameRate
        self.framePeriod = 1 / self.frameRate

    def getFrameRate(self):
        """
        Returns the frame rate of the frame.

        Returns:
            int or float: The frame rate of the frame.
        """
        return self.frameRate

    def setFrameRate(self, fr):
        """
        Sets the frame rate of the frame.

        Args:
            fr (int or float): The new frame rate for the frame.
        """
        self.frameRate = fr

    def getFramePeriod(self):
        """
        Returns the time duration of each frame.

        Returns:
            float: The time duration of each frame in seconds.
        """
        return 1 / self.frameRate

    def setFramePeriod(self, fp):
        """
        Sets the time duration of each frame.

        Args:
            fp (float): The new frame period for the frame in seconds.
        """
        self.framePeriod = fp



import os
import sys
sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), '..')))

from seeFretboard import SeeFretboard
from bokeh.layouts import row
from bokeh.io import  curdoc


fretboard1 = SeeFretboard()  
fretboard1.drawHorizontalFretboard()
fretboard2 = SeeFretboard("v")  
fretboard2.drawVerticalFretboard()

layout = row(fretboard1.layout, fretboard2.layout)  
curdoc().add_root(layout)
